# Flutter SDK

## ¿Como publicar?

- Cambiar version de release en `pubspec.yaml`
- Cambiar version el en atributo `sdk_version` de `request_behaviour.dart`
- Ejecutar los test unitarios
```
// En el directorio raiz del proyecto
flutter test

// Y en el app de ejemplo
cd example
flutter test
```
- Ejecutar los test de integracion
```
// Se debe tener activo al menos un emulador (android/ios)
// Navegar hacia el app de ejemplo, en ella estan las pruebas de integracion
cd example

flutter test integration_test
```
- Agregar al CHANGELOG.md las descripciones del cambio
- Crear un commit con los cambios y publicar
- Ejecutar `flutter pub publish`

---

## ¿Cómo preparar el ambiente de desarrollo?

### Antes de empezar

Esta guía esta hecha para macOS y basada en [la documentación oficial de Flutter](https://docs.flutter.dev/get-started/install/macos)

---

### Preparación del ambiente

### Terminal (Parte 1)

- Según la documentación, si se tiene una “Silicon Mac” (M1) se debe ejecutar este comando para poder ejecutar aplicaciones diseñadas para la arquitectura Intel x86 en la arquitectura ARM:

```dart
sudo softwareupdate --install-rosetta --agree-to-license
```

- Instalar el SDK Flutter

```dart
brew install --cask flutter
```

- Ejecutar el siguiente comando para verificar que todo esta listo, en caso de que falten dependencias se mostrara cuales faltan con sus respectivas instrucciones para instalarlas, una vez instalado todo verificar corriendo el comando nuevamente

```dart
flutter doctor
```

---
#### Ejemplo
Aqui se obtuvo este resultado:

![image info](./docs/img/flutter_doctor_example1.png)

Contexto: En esta máquina ya se tenía instalado Android Studio, pero faltaban algunos detalles.

- Para solucionar el primer error `cmdline-tools component is missing` se tuvo que instalar un paquete faltante, el cual se muestra en la siguiente imagen:

![image info](./docs/img/flutter_doctor_example2.png)

- Para solucionar el segundo error `Android license status unknown`, se utilizo el comando recomendado en la misma linea de comandos.

```dart
flutter doctor --android-license
```

> 💡 Al final de esta documentación, se mostrará una lista completa de todas las dependencias del “Android SDK & Tools” usadas por el IDE instaladas en la máquina de ejemplo hasta el momento.
---

### Android Studio (Parte 2)

- Una vez instalado y ejecutado Android Studio, utiliza la combinación de teclas `⌘ + ,` y esto abrirá las preferencias/ajustes.
- Seleccionar el apartado de ***Plugins***, y luego ***Marketplace*** y buscar `flutter` para poder instalar el plugin oficial. También se pedirá que se instale `Dart` y finalmente reiniciar el IDE.
- En el mismo menú revisar el apartado de ***Languages & Frameworks>Flutter*** para identificar si el path es correcto, sino configurarlo; igualmente con ***Dart***. El siguiente comando es para identificar el path:

```dart
where flutter
```

---
#### 💢 Errores

Tras Obtener el SDK de Flutter (Siguiente paso en esta guía) se deberían poder ejecutar los emuladores de Android e iOS desde Android Studio.

![image info](./docs/img/flutter_emulation_error_example1.png)

- En caso de tener problemas con el emulador de Android se debe revisar el resultado en consola para obtener un mejor contexto y solucionar el problema.
- En caso de tener problemas con el emulador de iOS se puede intentar con el siguiente comando, el cual fue extraído de la documentación de Flutter.

```dart
sudo gem uninstall ffi && sudo gem install ffi -- --enable-libffi-alloc
```

- Si aún arroja un error referente a “no se pudo ejecutar pod install” se puede abrir Android Studio desde la consola. De esta manera, las terminales deberían sincronizarse.

```dart
open /Applications/Android\ Studio.app
```
---

### Obtener SDK de Flutter

> 💡 [Enlace al repositorio de Flutter](https://bitbucket.org/pixelpay/sdk-flutter/src/master/)

Aprovechando el enlace anterior se puede clonar el repositorio. Usualmente, este se guarda en `~/Sites`. Además, se debe clonar el submódulo del pod (para tenerlo disponible en el apartado de IOS) con los siguientes comandos:

```dart
cd sdk-flutter
git submodule init
git submodule update
```

---

#### Comandos útiles

```dart
// Para revisar si no falta alguna dependencia clave de la instalación de
// flutter
flutter doctor

// Para obtener todas las dependencias
flutter packages get

// Para ejecutar los test unitarios
flutter test

// Para ejecutar los test de integración
cd example
flutter test integration_test

// Para limpiar las build hechas por flutter
flutter clean

// Para ejecutar un emulador desde la terminal en caso de que no sea
// posible desde Android Studio (se debe navegar hasta el directorio que
// contiene a lib/main.dart)
flutter build apk --debug
flutter build ios --simulator

// Para crear las versiones productivas
flutter build apk --release
flutter build ios --release

// Se debe tener el emluador que corresponde en ejecucion
flutter run

// Ejecutar la version release en Android
flutter run --release
```

---

#### 💡 Lista completa de dependencias Android SDK & Tools

![SDK Platforms](./docs/img/flutter_android_sdk_platforms.png "SDK Platforms")
SDK Platforms

![SDK Tools](./docs/img/flutter_android_sdk_tools.png "SDK Tools")
SDK Tools