import 'package:flutter/material.dart';
import 'package:flutter_test/flutter_test.dart';
import 'package:integration_test/integration_test.dart';

import 'package:pixelpay_sdk_example/main.dart' as app;

void main() {
  IntegrationTestWidgetsFlutterBinding.ensureInitialized();

  /// Global variable for waiting attempts, 20 * 500 milliseconds = 10 seconds
  int maxAttempts = 20;

  /// Global variable for counting attempts
  int attempt = 0;

  /// Evaluates the remaining attempts
  bool attemptsLeft() {
    if (attempt >= maxAttempts) {
      return false;
    }

    return true;
  }

  /// Evaluates if a text exists
  bool findText(String text) {
    if (find.text(text).evaluate().isNotEmpty) {
      return true;
    }

    return false;
  }

  // Check if the response has changed
  Future<void> checkResponseChange(tester) async {
    while (attemptsLeft() && findText('Empty')) {
      await tester.pump(const Duration(milliseconds: 500));
      attempt += 1;
    }
  }

  testWidgets('testSuccessSaleTransaction', (WidgetTester tester) async {
    // Start the application and wait for it
    app.main();
    await tester.pumpAndSettle();

    // Send the transaction
    await tester.tap(find.byKey(const Key('SALE')));
    await tester.pumpAndSettle();

    attempt = 0;
    await checkResponseChange(tester);

    // Search for the success response
    expect(tester.widget<Text>(find.byKey(const Key('status_code'))).data, '200');
    expect(tester.widget<Text>(find.byKey(const Key('main_message'))).data, 'Payment sent successfully');

    await tester.tap(find.byKey(const Key('RESET')));
  });

  testWidgets('testSuccessfulAuthentication', (WidgetTester tester) async {
    app.main();
    await tester.pumpAndSettle();

    await tester.tap(find.byKey(const Key('Select card')));
    await tester.pumpAndSettle();

    await tester.tap(find.text('(2.1) Successful Authentication'));
    await tester.pumpAndSettle();

    await tester.tap(find.byKey(const Key('SALE')));
    await tester.pumpAndSettle();

    attempt = 0;
    await checkResponseChange(tester);

    expect(tester.widget<Text>(find.byKey(const Key('status_code'))).data, '200');
    expect(tester.widget<Text>(find.byKey(const Key('main_message'))).data, 'Payment sent successfully');

    await tester.tap(find.byKey(const Key('RESET')));
  });

  testWidgets('testFailedAuthentication', (WidgetTester tester) async {
    app.main();
    await tester.pumpAndSettle();

    await tester.tap(find.byKey(const Key('Select card')));
    await tester.pumpAndSettle();

    await tester.tap(find.text('(2.2) Failed Authentication'));
    await tester.pumpAndSettle();

    await tester.tap(find.byKey(const Key('SALE')));
    await tester.pumpAndSettle();

    attempt = 0;
    await checkResponseChange(tester);

    expect(tester.widget<Text>(find.byKey(const Key('status_code'))).data, '402');
    expect(tester.widget<Text>(find.byKey(const Key('main_message'))).data, 'Transaction denied');

    await tester.tap(find.byKey(const Key('RESET')));
  });

  testWidgets('testAttemptsStandInAuthentication', (WidgetTester tester) async {
    app.main();
    await tester.pumpAndSettle();

    await tester.tap(find.byKey(const Key('Select card')));
    await tester.pumpAndSettle();

    await tester.tap(find.text('(2.3) Attempts Stand-In Authentication'));
    await tester.pumpAndSettle();

    await tester.tap(find.byKey(const Key('SALE')));
    await tester.pumpAndSettle();

    attempt = 0;
    await checkResponseChange(tester);

    expect(tester.widget<Text>(find.byKey(const Key('status_code'))).data, '200');
    expect(tester.widget<Text>(find.byKey(const Key('main_message'))).data, 'Payment sent successfully');

    await tester.tap(find.byKey(const Key('RESET')));
  });

  testWidgets('testUnavailableAuthenticationIssuer', (WidgetTester tester) async {
    app.main();
    await tester.pumpAndSettle();

    await tester.tap(find.byKey(const Key('Select card')));
    await tester.pumpAndSettle();

    await tester.tap(find.text('(2.4) Unavailable Authentication from the Issuer'));
    await tester.pumpAndSettle();

    await tester.tap(find.byKey(const Key('SALE')));
    await tester.pumpAndSettle();

    attempt = 0;
    await checkResponseChange(tester);

    expect(tester.widget<Text>(find.byKey(const Key('status_code'))).data, '402');
    expect(tester.widget<Text>(find.byKey(const Key('main_message'))).data, 'Transaction denied');

    await tester.tap(find.byKey(const Key('RESET')));
  });

  testWidgets('testRejectedAuthenticationIssuer', (WidgetTester tester) async {
    app.main();
    await tester.pumpAndSettle();

    await tester.tap(find.byKey(const Key('Select card')));
    await tester.pumpAndSettle();

    await tester.tap(find.text('(2.5) Rejected Authentication by the Issuer'));
    await tester.pumpAndSettle();

    await tester.tap(find.byKey(const Key('SALE')));
    await tester.pumpAndSettle();

    attempt = 0;
    await checkResponseChange(tester);

    expect(tester.widget<Text>(find.byKey(const Key('status_code'))).data, '402');
    expect(tester.widget<Text>(find.byKey(const Key('main_message'))).data, 'Transaction denied');

    await tester.tap(find.byKey(const Key('RESET')));
  });

  testWidgets('testAuthenticationNotAvailable', (WidgetTester tester) async {
    app.main();
    await tester.pumpAndSettle();

    await tester.tap(find.byKey(const Key('Select card')));
    await tester.pumpAndSettle();

    await tester.tap(find.text('(2.6) Authentication Not Available on Lookup'));
    await tester.pumpAndSettle();

    await tester.tap(find.byKey(const Key('SALE')));
    await tester.pumpAndSettle();

    attempt = 0;
    await checkResponseChange(tester);

    expect(tester.widget<Text>(find.byKey(const Key('status_code'))).data, '402');
    expect(tester.widget<Text>(find.byKey(const Key('main_message'))).data, 'Transaction denied');

    await tester.tap(find.byKey(const Key('RESET')));
  });

  testWidgets('testBypassedAuthentication', (WidgetTester tester) async {
    app.main();
    await tester.pumpAndSettle();

    await tester.tap(find.byKey(const Key('Select card')));
    await tester.pumpAndSettle();

    await tester.tap(find.text('(2.13) Bypassed Authentication'));
    await tester.pumpAndSettle();

    await tester.tap(find.byKey(const Key('SALE')));
    await tester.pumpAndSettle();

    attempt = 0;
    await checkResponseChange(tester);

    expect(tester.widget<Text>(find.byKey(const Key('status_code'))).data, '402');
    expect(tester.widget<Text>(find.byKey(const Key('main_message'))).data, 'Transaction denied');

    await tester.tap(find.byKey(const Key('RESET')));
  });

  testWidgets('testRequireMethodURL', (WidgetTester tester) async {
    app.main();
    await tester.pumpAndSettle();

    await tester.tap(find.byKey(const Key('Select card')));
    await tester.pumpAndSettle();

    await tester.tap(find.text('(2.14) Require MethodURL'));
    await tester.pumpAndSettle();

    await tester.tap(find.byKey(const Key('SALE')));
    await tester.pumpAndSettle();

    attempt = 0;
    await checkResponseChange(tester);

    expect(tester.widget<Text>(find.byKey(const Key('status_code'))).data, '402');
    expect(tester.widget<Text>(find.byKey(const Key('main_message'))).data, 'Transaction denied');

    await tester.tap(find.byKey(const Key('RESET')));
  });
}
// TODO: test por fabricar
// "success":true
// "success":false                                    
// (2.7) Error on Lookup                              500   [1001] Error Processing Message Request
// (2.8) Timeout on cmpi_lookup Transaction           500   [2860] Error Processing Message Request with Directory Server
// (2.9) Successful Step Up Authentication            !!!
// (2.10) Failed Step Up Authentication               !!!
// (2.11) Step Up Authentication is Unavailable       !!!
// (2.12) Error on Authentication                     !!!