import 'dart:convert';

import 'package:flutter/services.dart';

class Locations {
	/// Countries resource repository
	static Map<String, dynamic> _countriesRepository = Map();

	/// States resource repository
	static Map<String, dynamic> _statesRepository = Map();

  /// Formats resource repository
	static Map<String, dynamic> _formatsRepository = Map();

	/// Verify respository is available
	static Future<void> _checkRepository() async {
		if (Locations._countriesRepository.isEmpty || Locations._statesRepository.isEmpty || Locations._formatsRepository.isEmpty) {
      String countries = await rootBundle.loadString('packages/pixelpay_sdk/assets/countries.json', cache: true);
      String states = await rootBundle.loadString('packages/pixelpay_sdk/assets/states.json', cache: true);
      String formats = await rootBundle.loadString('packages/pixelpay_sdk/assets/formats.json', cache: true);

      _countriesRepository = jsonDecode(countries);
      _statesRepository = jsonDecode(states);
      _formatsRepository = jsonDecode(formats);
		}
	}

	/// Return a list of countries
	/// 
	/// @return
	static Future<Map<String, dynamic>> countriesList() async {
		await _checkRepository();

    final countriesList = _countriesRepository;
		return countriesList;
	}

	/// Get states list by country ISO code
	/// 
	/// @param country_code
	/// @return
	static Future<Map<String, dynamic>> statesList(String country_code) async {
	 await _checkRepository();

    if (!_statesRepository.containsKey(country_code)) {
      return {};
    }

		return _statesRepository[country_code];
	}

  /// Get phone and zip formats list by country ISO code
	/// 
	/// @param country_code
	/// @return
	static Future<Map<String, dynamic>> formatsList(String country_code) async {
		await _checkRepository();

    if (!_formatsRepository.containsKey(country_code)) {
      return {};
    }

		return _formatsRepository[country_code];
	}
}
